"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const get_stdin_1 = __importDefault(require("get-stdin"));
const read_pkg_1 = __importDefault(require("read-pkg"));
const getConf_1 = __importDefault(require("../getConf"));
function runCmd(cwd, hookName, cmd, env) {
    const shellPath = process.env.SHELL || 'sh';
    let status;
    console.log(`husky > ${hookName} (node ${process.version})`);
    try {
        status = child_process_1.spawnSync(shellPath, ['-c', cmd], {
            cwd,
            env: Object.assign(Object.assign({}, process.env), env),
            stdio: 'inherit'
        }).status;
    }
    catch (_a) {
        status = 1;
    }
    if (status !== 0) {
        const noVerifyMessage = [
            'commit-msg',
            'pre-commit',
            'pre-rebase',
            'pre-push'
        ].includes(hookName)
            ? '(add --no-verify to bypass)'
            : '(cannot be bypassed with --no-verify due to Git specs)';
        console.log(`husky > ${hookName} hook failed ${noVerifyMessage}`);
    }
    return status || 0;
}
/**
 * @param {array} argv - process.argv
 * @param {string} options.cwd - cwd
 * @param {promise} options.getStdinFn - used for mocking only
 */
function run([, , hookName = '', HUSKY_GIT_PARAMS], { cwd = process.cwd(), getStdinFn = get_stdin_1.default } = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        // In some cases, package.json may not exist
        // For example, when switching to gh-page branch
        let pkg;
        try {
            pkg = read_pkg_1.default.sync({ cwd, normalize: false });
        }
        catch (err) {
            if (err.code !== 'ENOENT') {
                throw err;
            }
        }
        const config = getConf_1.default(cwd);
        const command = config && config.hooks && config.hooks[hookName];
        const oldCommand = pkg && pkg.scripts && pkg.scripts[hookName.replace('-', '')];
        // Add HUSKY_GIT_PARAMS to env
        const env = {};
        if (HUSKY_GIT_PARAMS) {
            env.HUSKY_GIT_PARAMS = HUSKY_GIT_PARAMS;
        }
        // Read stdin
        if (['pre-push', 'pre-receive', 'post-receive', 'post-rewrite'].includes(hookName)) {
            // Add HUSKY_GIT_STDIN to env
            env.HUSKY_GIT_STDIN = yield getStdinFn();
        }
        if (oldCommand) {
            console.log(`
Warning: Setting ${hookName} script in package.json > scripts will be deprecated.
Please move it to husky.hooks in package.json or .huskyrc file.

For an automatic updata you can also run:
npx --no-install husky-upgrade
yarn husky-upgrade

See https://github.com/typicode/husky for more information.
`);
        }
        if (command) {
            return runCmd(cwd, hookName, command, env);
        }
        if (oldCommand) {
            return runCmd(cwd, hookName, oldCommand, env);
        }
        return 0;
    });
}
exports.default = run;
